#include <kate/application.h>
#include <kate/documentmanager.h>
#include <kate/document.h>
#include <kate/mainwindow.h>
#include <qlineedit.h>
#include <qtextcodec.h>
#include <qtextstream.h>
#include <qtextedit.h>
#include <qlistview.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qpopupmenu.h>

#include "cppdevelop.utils.h"
#include "class.objectmap.h"
#include "class.cppviewitem.h"
#include "class.cppview.h"
#include "class.cppview.moc"
#include "class.absparser.h"

/*!
 * @fn CppView::_draw
 * @brief Description
 * draw view after create
 * @return void
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
void CppView::_draw() {
    setSorting(-1);
    setShowToolTips(TRUE);
    addColumn(QString("element"));
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    //setResizeMode(QListView::AllColumns);
    setColumnWidthMode(0,QListView::Maximum);
    objects = NULL;
    parser = NULL;
    menu = new QPopupMenu(this);
    menu->insertItem( QString("generate doxy document"),0 );
    menu->insertItem( QString("refresh list"),1 );
    reDraw();
}

/*!
 * @fn CppView::reDraw
 * @return void
 * @brief Description
 * redraw cppview
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
void CppView::reDraw() {
    clearSafe();
    _macro = new QListViewItem(this, lastItem(), QString("Macros"));
    _struct = new QListViewItem(this, lastItem(), QString("Structures"));
    _class = new QListViewItem(this, lastItem(), QString("Class"));
    _func = new QListViewItem(this, lastItem(), QString("Functions"));
    QPixmap cls( ( const char** ) class_xpm );
    QPixmap sct( ( const char** ) struct_xpm );
    QPixmap mcr( ( const char** ) macro_xpm );
    _macro->setPixmap(0, (const QPixmap &)mcr);
    _func->setPixmap(0, (const QPixmap &)sct);
    _struct->setPixmap(0, (const QPixmap &)cls);
    _class->setPixmap(0, (const QPixmap &)cls);
    setRootIsDecorated(1);
    objects = ObjectMap::Instance();
}

/*!
 * @fn CppView::~CppView
 * @brief Constructor
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
CppView::~CppView() {
    clearSafe();
}

void CppView::slotSearch() {
        if(!searchText) return;
        bool viewAll = searchText->text().isEmpty();
        QListViewItem * myChild = NULL;
        QListViewItem * myChildClass = NULL;
#define FIND_IN_BLOCK(block)   if(block && block->childCount() && !block->isOpen()) block->setOpen(TRUE); \
                                         myChild = block->firstChild();\
                                         while( myChild  ) {\
                                                if(myChild->childCount()) {\
                                                   if(!myChild->isOpen()) myChild->setOpen(TRUE);\
                                                   myChildClass = myChild->firstChild();\
                                                   while( myChildClass ) {\
                                                        myChildClass->setVisible((myChildClass->text(0).find(searchText->text()) >= 0 || viewAll));\
                                                        myChildClass = myChildClass->nextSibling();\
                                                   }\
                                                   myChild = myChild->nextSibling();\
                                                } else {\
                                                    myChild->setVisible((myChild->text(0).find(searchText->text()) >= 0 || viewAll));\
                                                    myChild = myChild->nextSibling();\
                                                }\
                                         }
           FIND_IN_BLOCK(_macro);
           FIND_IN_BLOCK(_class);
           FIND_IN_BLOCK(_struct);
           FIND_IN_BLOCK(_func);
}

/*!
 * @fn CppView::registerWin
 * @param Kate::MainWindow *winDoc
 * @return void
 * @brief Description
 * Register Kate::windows in this class
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
void CppView::registerWin(Kate::MainWindow *winDoc) {
      win = winDoc;
      connect( this, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(slotGotoLine(QListViewItem *)));
      connect( this, SIGNAL(selectionChanged ( QListViewItem * )), this, SLOT(slotShowDoc(QListViewItem *)));
      connect( this, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint&, int)), this, SLOT(slotShowContextMenu(QListViewItem *, const QPoint&, int)));
      connect(win->viewManager(), SIGNAL(viewChanged()), this, SLOT(slotDocChanged()));
      connect(menu, SIGNAL(activated( int )), this, SLOT(slotMenuExec( int  )));
}

/*!
 * @fn CppView::clearSafe
 * @return void
 * @brief Description
 * Clear all element
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
void CppView::clearSafe()
{
    clear();
    if(objects) objects->FreeInst();
    objects = NULL;
    _macro = NULL;
    _func = NULL;
    _struct = NULL;
    _class = NULL;
}

/*!
 * @fn CppView::getItem
 * @param DefType type
 * @return QListViewItem*
 * @brief Description
 * Get Group element Items
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
QListViewItem* CppView::getItem( DefType type )
{
    switch(type) {
        case Func:
            return _func;
            break;
        case Class:
            return _class;
            break;
        case Macro:
            return _macro;
            break;
        case Struct:
            return _struct;
            break;
    };
}

/*!
 * @fn CppView::slotShowDoc
 * @param QListViewItem *it
 * @return void
 * @brief Description
 * Show Description element
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
void CppView::slotShowDoc(QListViewItem *it) {
            if(CHECK_ITEM(it)) { return; }
            ItemObject* itObj = FIND_OBJ(it);
            QString txt;
            if(itObj)  txt.append(itObj->getDocText());
            doxyText->setText( txt );
}

/*!
 * @fn CppView::slotMenuExec
 * @param int  id
 * @return void
 * @brief SLOT for contextmenu (Interface for parser::createDoc)
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
void CppView::slotMenuExec( int  id ) {
      if(id == 1) {
                      slotDocChanged();
                      return;
      }
      QListViewItem *it = selectedItem();
      if(CHECK_ITEM(it)) return;
      ItemObject* itObj = FIND_OBJ(it);

      if(!itObj) return;
      QString *nameDoc =&(((CppViewItemBase *)it)->nameId);
      DefType typeDoc = ((CppViewItemBase *)it)->type;

      QString DoxyComment = parser->createDoc(*nameDoc, typeDoc);

      if(DoxyComment.isEmpty()) return;
      // insert doc in document
      Kate::Document *kv = win->viewManager()->activeView()->getDoc();
      kv->insertText(itObj->getStrNum(), 0, DoxyComment);
      DoxyObj *dxy = new DoxyObj(*nameDoc, DoxyComment);
      dxy->type = typeDoc;
      itObj->setDoc(dxy);
      slotShowDoc(it);
      slotGotoLine(it);
      slotDocChanged();
}

/*!
 * @fn CppView::slotGotoLine
 * @param QListViewItem *it
 * @return void
 * @brief goto definition element in doc
 *
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
void CppView::slotGotoLine(QListViewItem *it) {
            if(CHECK_ITEM(it)) return;
            ItemObject* itObj = FIND_OBJ(it);
            if(itObj) {
                        Kate::View *kv = win->viewManager()->activeView();
                        if (!kv)
                            return;
                kv->gotoLineNumber(itObj->getStrNum() + 10);
                kv->setFocus();
                kv->gotoLineNumber(itObj->getStrNum());
            }
}

/*!
 * @fn CppView::slotShowContextMenu
 * @param QListViewItem *it
 * @param const QPoint &p
 * @param int
 * @return void
 * @brief show context menu
 *
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
void CppView::slotShowContextMenu(QListViewItem *it, const QPoint &p, int){
        if(CHECK_ITEM(it)) {
            // show only Refresh contextmenu Item
            menu->setItemEnabled(0,FALSE);
        } else {
            ItemObject* itObj = FIND_OBJ(it);
            if(!itObj || itObj->getDocText().isEmpty()) {
                      menu->setItemEnabled(0,TRUE);
            }else{
                      menu->setItemEnabled(0,FALSE);
            }
        }
        menu->exec(p);
}

/*!
 * @fn CppView::slotDocChanged
 * @return void
 * @brief run if document view changed
 * @author Andrei Dmitriev <andrei_dm@mail.ru>
 */
void CppView::slotDocChanged() {
        reDraw();
        unsigned int hlMode = 0;
        if (!win->viewManager()->activeView()) return;

        Kate::Document *kv = win->viewManager()->activeView()->getDoc();
        if (!kv)   return;

        hlMode = kv->hlMode();
        QString hlModeName = kv->hlModeName(hlMode);
        if(parser) delete parser;
        parser = NULL;
        if (hlModeName == "C++" || hlModeName == "C")
                    parser = new CppParser();
        else if ( hlModeName == "Perl" )
                parser = new PerlParser();
        else {
                parser = new JavaScriptParser();
        }
       if(parser)  parser->parse(kv);
       doxyText->setText( QString("") );
}


