#ifndef _CLASS_CPPVIEW_H_
#define _CLASS_CPPVIEW_H_



#include <kate/application.h>
#include <kate/documentmanager.h>
#include <kate/document.h>
#include <kate/mainwindow.h>

#include <qlistview.h>
#include <qlineedit.h> 
#include "class.singleton.h"

#include "plugin_katecppdevelop.h"

#define CHECK_ITEM(it)  !it || it == _func || it == _class || it == _macro || it == _struct
#define FIND_OBJ(it) objects->findObject(((CppViewItemBase *)it)->nameId, ((CppViewItemBase *)it)->type)

class ObjectMap;
class QTextEdit;
class AbsParser;
class QPopupMenu;

/*!
 * @class CppView : public QListView, public Singleton<CppView>
 * @brief detail description
 * 
 * @author Andrei Dmitriev <andrei_dm@mail.ru> 
 *
 */
class CppView : public QListView, public Singleton<CppView>
{
      Q_OBJECT
    private:
       QListViewItem *_func;
       QListViewItem *_macro;
       QListViewItem *_class;
       QListViewItem *_struct;
       QLineEdit *searchText;
       QTextEdit *doxyText;
       Kate::MainWindow *win;
       ObjectMap *objects;
       AbsParser *parser;
       QPopupMenu *menu;
    public:
        enum DefType { Func=0, Class, Macro, Struct } Type;
        QListViewItem* getItem( DefType type );
        void clearSafe();
        void reDraw();
        void registerWin(Kate::MainWindow *winDoc);
        void setSearchField(QLineEdit *sEdit) { searchText = sEdit; }
        void setDoxyField(QTextEdit *txt) { doxyText = txt; }
    public slots:
        void slotGotoLine(QListViewItem *);
        void slotShowDoc(QListViewItem *);
        void slotShowContextMenu(QListViewItem *, const QPoint&, int);
        void slotDocChanged();
        void slotSearch();
        void slotMenuExec( int  id );    
    protected:
       void _draw();
       CppView(QWidget * parent = 0):QListView( parent ),searchText(NULL) { _draw(); }
        ~CppView();
    friend class Singleton<CppView>;
};

#endif