#ifndef OBJECT_MAP_H
#define OBJECT_MAP_H

#include <qmap.h>
#include <qstring.h>
#include "class.singleton.h"
#include "class.cppviewitem.h"
#include "class.cppview.h"

class DoxyObj
{
      QString *name;
      QString *text;
public:
      CppView::DefType type;
      DoxyObj(QString itName, QString sText):name(new QString(itName)),text(new QString(sText)) {}
      DoxyObj():name(NULL),text(NULL) {}
      ~DoxyObj() { if(name) delete name; if(text) delete text; }
      QString getText() { return (text) ? *text : QString(""); }
      QString getName() const { return (text) ? *name : QString(""); }
      void setName( QString itName ) { if(name) delete name; name = &itName; }
      void setText( QString itText ) { if(text) delete text; text = &itText; }
};

class ItemObject
{
    protected:
         int strNum;
         QString filename;
         QString pattern;
         DoxyObj *doxygen;
    public:
        CppViewItemBase* viewItem;
        ItemObject(CppView::DefType type, QString &itName);
        ItemObject(CppView::DefType type, QString &itName, QString &showName, ItemObject *parent);
        ~ItemObject();
        QString *getName() { return &(viewItem->nameId);  }
        void setDoc(DoxyObj *doc) { if(doxygen) delete  doxygen; doxygen = doc; }
        QString getDocText() { return (doxygen) ? doxygen->getText() : QString("");  }
        void setStrNum( uint num ) { strNum = num; }
        int getStrNum() { return  strNum; }
};

typedef QMap<QString,ItemObject *> ItemMap;
typedef QMap<CppView::DefType,  ItemMap *> ItemsMap;

class ObjectMap : public Singleton<ObjectMap>
{
    private:
         ItemsMap items;
    public:
         ItemObject *addObject(QString &name, CppView::DefType type);
         ItemObject *addObject(QString &name,  QString &showName, CppView::DefType type, ItemObject *parent);
         ItemObject *findObject(QString &name,  CppView::DefType type);
         ItemObject *wordInObject(const QString &name,  CppView::DefType type);
    protected:
        ObjectMap();
        ~ObjectMap();
    friend class Singleton<ObjectMap>;
};

#endif // OBJECT_MAP_H
