#include "plugin_katecppdevelop.h"
#include "plugin_katecppdevelop.moc"

#include <kaction.h>
#include <klocale.h>
#include <kgenericfactory.h>
#include <ksyntaxhighlighter.h>

#include <qsplitter.h>
#include <qlistview.h>
#include <qtextedit.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include "class.cppview.h"

K_EXPORT_COMPONENT_FACTORY( katecppdevelopplugin, KGenericFactory<KateCppDevelopPlugin>( "katecppdevelop" ) );

KateCppDevelopView::KateCppDevelopView(Kate::MainWindow *w)
{
    KGlobal::locale()->insertCatalogue("katecppdevelop");
    setInstance (new KInstance("kate"));
    setXMLFile("plugins/katecppdevelop/ui.rc");
    w->guiFactory()->addClient (this);
    win = w;
    _draw();
}

KateCppDevelopView::~KateCppDevelopView()
{
  win->guiFactory()->removeClient (this);
  elemList->FreeInst();
  delete txt;
  delete ledt;
  delete btn;
  delete lbl;
  delete hbox;
  delete vbox;  
  delete split;
  delete mainWidget;
}

/*!
 * @fn KateCppDevelopView::_draw
 * @return void
 * @brief Description
 * Draw interface
 * @author Andrei Dmitriev <andrei_dm@mail.ru> 
 */
void KateCppDevelopView::_draw()
{
  QPixmap cls( ( const char** ) class_xpm );
  mainWidget = win->toolViewManager()->createToolView("kate_plugin_cppdevelop", Kate::ToolViewManager::Left, cls, QString("C/C++ Develop"));
  split = new QSplitter( Qt::Vertical, mainWidget );
  elemList = CppView::QInstance( split ); // Element List
  elemList->registerWin(win);
  QWidget *button = new QWidget(split);
  button->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  vbox = new QVBoxLayout(button); // VBox
  hbox = new QHBoxLayout(vbox); //HBox
  ledt   = new QLineEdit(button);
  ledt->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Expanding);
  ledt->setFixedHeight(20);
  hbox->addWidget(ledt);
  btn = new QPushButton(button);
  btn->setFixedHeight(20);
  btn->setText(QString("Search"));
  btn->setFixedWidth(70);
  btn->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
  hbox->addWidget(btn);  
  lbl = new QLabel(QString("Doxy document"), button);
  lbl->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
  lbl->setFixedHeight(20);
  vbox->addWidget(lbl);
  txt = new QTextEdit(button);
  txt->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);  
  //txt->setTextFormat(Qt::PlainText);
  ksintax = new KSyntaxHighlighter(txt,TRUE);
  txt->setReadOnly(TRUE);
  vbox->addWidget(txt);
  elemList->setSearchField(ledt);
  elemList->setDoxyField(txt);
  QObject::connect(ledt, SIGNAL(returnPressed()), elemList, SLOT(slotSearch())); 
  QObject::connect(btn, SIGNAL(clicked()), elemList, SLOT(slotSearch()));
}

KateCppDevelopPlugin::KateCppDevelopPlugin( QObject* parent, const char* name, const QStringList& )
    : Kate::Plugin ( (Kate::Application*)parent, name )
{
}

KateCppDevelopPlugin::~KateCppDevelopPlugin()
{
}

/*!
 * @fn KateCppDevelopPlugin::addView
 * @param Kate::MainWindow *win
 * @return void
 * @brief create plugin in interface
 * @author Andrei Dmitriev <andrei_dm@mail.ru> 
 */
void KateCppDevelopPlugin::addView (Kate::MainWindow *win)
{
  KateCppDevelopView *view = new KateCppDevelopView (win);
  m_views.append (view);
  
}

/*!
 * @fn KateCppDevelopPlugin::removeView
 * @param Kate::MainWindow *win
 * @return void
 * @brief remove Plugin from Interface
 * @author Andrei Dmitriev <andrei_dm@mail.ru> 
 */
void KateCppDevelopPlugin::removeView(Kate::MainWindow *win)
{
  for (uint z=0; z < m_views.count(); z++)
    if (m_views.at(z)->win == win)
    {
      KateCppDevelopView *view = m_views.at(z);
      m_views.remove (view);
      delete view;
      return;
    }
}


