#ifndef _PLUGIN_KATE_CPPDEVELOP_H_
#define _PLUGIN_KATE_CPPDEVELOP_H_

#include <kate/application.h>
#include <kate/documentmanager.h>
#include <kate/document.h>
#include <kate/mainwindow.h>
#include <kate/plugin.h>
#include <kate/view.h>
#include <kate/viewmanager.h>
#include <kate/toolviewmanager.h>
#include <kate/pluginconfiginterface.h>
#include <kate/pluginconfiginterfaceextension.h>

#include <kdebug.h>
#include <qmemarray.h>
#include <qpopupmenu.h>
#include <qevent.h>
#include <qcheckbox.h>
#include <klibloader.h>
#include <klocale.h>
#include <klistview.h>
#include <kiconloader.h>
#include <kconfig.h>

class KSyntaxHighlighter;
class CppView;
class QSplitter;
class QVBoxLayout;
class QHBoxLayout;
class QLabel;
class QTextEdit;
class QLineEdit;
class QPushButton;

/*!
 * @class KateCppDevelopView : public KXMLGUIClient
 * @brief Description
 * main Widget plugin
 * @author Andrei Dmitriev <andrei_dm@mail.ru> 
 */
class KateCppDevelopView : public KXMLGUIClient
{
  private:
    QWidget *mainWidget;
    QTextEdit *txt;
    KSyntaxHighlighter *ksintax;
    QVBoxLayout *vbox;
    QHBoxLayout *hbox;
    QPushButton *btn; //!< for search word
    QLabel *lbl; //!< comment
    QLineEdit *ledt; //!< for search word
     void _draw();
  protected:
     CppView *elemList; 
  public:
    KateCppDevelopView (Kate::MainWindow *w);
    virtual ~KateCppDevelopView ();
    Kate::MainWindow *win;
    QSplitter *split;
};

/*!
 * @class KateCppDevelopPlugin : public Kate::Plugin, Kate::PluginViewInterface
 * @brief Description
 * main class plugin
 * @author Andrei Dmitriev <andrei_dm@mail.ru> 
 */
class KateCppDevelopPlugin : public Kate::Plugin, Kate::PluginViewInterface
{
  Q_OBJECT

  public:
    KateCppDevelopPlugin( QObject* parent = 0, const char* name = 0, const QStringList& = QStringList() );
    virtual ~KateCppDevelopPlugin();

    void addView (Kate::MainWindow *win);
    void removeView (Kate::MainWindow *win);

  public slots:
  
  private:
    QPtrList<KateCppDevelopView> m_views;
};

/*!
 * copy as i as from  cppsymbolviewer Kate plugin
 * @author               : 2003 Massimo Callegari
 * mail                : massimocallegari@yahoo.it
 */
static const char* const class_xpm[] = {
"16 16 10 1",
" 	c None",
".	c #000000",
"+	c #A4E8FC",
"@	c #24D0FC",
"#	c #001CD0",
"$	c #0080E8",
"%	c #C0FFFF",
"&	c #00FFFF",
"*	c #008080",
"=	c #00C0C0",
"     ..         ",
"    .++..       ",
"   .+++@@.      ",
"  .@@@@@#...    ",
"  .$$@@##.%%..  ",
"  .$$$##.%%%&&. ",
"  .$$$#.&&&&&*. ",
"   ...#.==&&**. ",
"   .++..===***. ",
"  .+++@@.==**.  ",
" .@@@@@#..=*.   ",
" .$$@@##. ..    ",
" .$$$###.       ",
" .$$$##.        ",
"  ..$#.         ",
"    ..          "};

static const char* const struct_xpm[] = {
"16 16 14 1",
" 	c None",
".	c #000000",
"+	c #C0FFC0",
"@	c #00FF00",
"#	c #008000",
"$	c #00C000",
"%	c #C0FFFF",
"&	c #00FFFF",
"*	c #008080",
"=	c #00C0C0",
"-	c #FFFFC0",
";	c #FFFF00",
">	c #808000",
",	c #C0C000",
"     ..         ",
"    .++..       ",
"   .+++@@.      ",
"  .@@@@@#...    ",
"  .$$@@##.%%..  ",
"  .$$$##.%%%&&. ",
"  .$$$#.&&&&&*. ",
"   ...#.==&&**. ",
"   .--..===***. ",
"  .---;;.==**.  ",
" .;;;;;>..=*.   ",
" .,,;;>>. ..    ",
" .,,,>>>.       ",
" .,,,>>.        ",
"  ..,>.         ",
"    ..          "};

// Sarah's colors...
static const char* const macro_xpm[] = {
"16 16 14 1",
" 	c None",
".	c #000000",
"+	c #FF7FE5",
"@	c #FF00C7",
"#	c #7F0066",
"$	c #BC0096",
"%	c #C0FFFF",
"&	c #00FFFF",
"*	c #008080",
"=	c #00C0C0",
"-	c #D493FF",
";	c #A100FF",
">	c #470082",
",	c #6B00B7",
"     ..         ",
"    .++..       ",
"   .+++@@.      ",
"  .@@@@@#...    ",
"  .$$@@##.%%..  ",
"  .$$$##.%%%&&. ",
"  .$$$#.&&&&&*. ",
"   ...#.==&&**. ",
"   .--..===***. ",
"  .---;;.==**.  ",
" .;;;;;>..=*.   ",
" .,,;;>>. ..    ",
" .,,,>>>.       ",
" .,,,>>.        ",
"  ..,>.         ",
"    ..          "};

#endif
